import { Brush } from "@mui/icons-material"
import TrashIcon from "@mui/icons-material/Delete"
import LockIcon from "@mui/icons-material/Lock"
import UnlockIcon from "@mui/icons-material/LockOpen"
import ReorderIcon from "@mui/icons-material/SwapVert"
import VisibleIcon from "@mui/icons-material/Visibility"
import VisibleOffIcon from "@mui/icons-material/VisibilityOff"
import { Tooltip } from "@mui/material"
import { grey } from "@mui/material/colors"
import Grid from "@mui/material/Grid"
import { createTheme, styled, ThemeProvider } from "@mui/material/styles"
import { makeStyles } from "@mui/styles"
import classnames from "classnames"
import React, { memo, useState } from "react"
import SidebarBoxContainer from "../SidebarBoxContainer"

import { asMutable } from "seamless-immutable"
import styles from "./styles"

const theme = createTheme()
const useStyles = makeStyles((theme) => styles)

const HeaderSep = styled("div")(({ theme }) => ({
  borderTop: `1px solid ${grey[200]}`,
  marginTop: 2,
  marginBottom: 2,
}))

const Chip = ({ color, text, index }) => {
  const classes = useStyles()

  return (
    <span
      className={classes.chip}
      onClick={() => window.brushHighlighted(index)}
    >
      <div className="color" style={{ backgroundColor: color }} />
      <div className="text">{text}</div>
    </span>
  )
}

const RowLayout = ({
  header,
  highlighted,
  order,
  classification,
  area,
  tags,
  trash,
  lock,
  visible,
  onClick,
}) => {
  const classes = useStyles()
  const [mouseOver, changeMouseOver] = useState(false)
  return (
    <div
      onClick={onClick}
      onMouseEnter={() => changeMouseOver(true)}
      onMouseLeave={() => changeMouseOver(false)}
      className={classnames(classes.row, { header, highlighted })}
    >
      <Grid container alignItems="center">
        <Grid item xs={2}>
          <div style={{ textAlign: "right", paddingRight: 10 }}>{order}</div>
        </Grid>
        <Grid item xs={5}>
          {classification}
        </Grid>
        <Grid item xs={2}>
          <div style={{ textAlign: "right", paddingRight: 6 }}>{area}</div>
        </Grid>
        <Grid item xs={1}>
          {trash}
        </Grid>
        <Grid item xs={1}>
          {lock}
        </Grid>
        <Grid item xs={1}>
          {visible}
        </Grid>
      </Grid>
    </div>
  )
}

const RowHeader = ({
  regions,
  brushLines,
  setLines,
  setdelete_annotation,
  delete_annotation,
}) => {
  let main_region = asMutable(regions || [])?.length
  let visible = asMutable(regions || [])?.filter(
    (item) => !item.isHide || item.isHide === undefined
  )

  let locked = asMutable(regions || [])?.every(
    (item) => item.isLocked === false || item.isLocked === undefined
  )
  const handleChangeRegion = (action = {}) => {
    let linessave = [...brushLines]
    // eslint-disable-next-line no-unused-expressions
    let final = linessave?.map((r) => Object.assign({ ...r, ...action }))

    setLines(final)
  }
  return (
    <RowLayout
      header
      highlighted={false}
      order={<ReorderIcon className="icon" />}
      classification={<div style={{ paddingLeft: 10 }}>Class</div>}
      trash={
        <Tooltip title="Delete" placement="right">
          <TrashIcon
            className="icon"
            onClick={() => {
              let arr = [...delete_annotation]

              setLines([])
              arr.push(regions)
              setdelete_annotation(arr.flat())
              window.undoArray = [...window.undoArray, "brush"]
              let obj = { brush: arr.flat() }
              let newArr = [...window.annotation_redo]
              if (obj.brush !== undefined) {
                newArr.push(obj)
                window.annotation_redo = newArr
              }
            }}
          />
        </Tooltip>
      }
      lock={
        <Tooltip title="Lock/Unlock" placement="right">
          {locked ? (
            <UnlockIcon
              className="icon"
              onClick={() => {
                // eslint-disable-next-line no-unused-expressions
                handleChangeRegion({ isLocked: true })
              }}
            />
          ) : (
            <LockIcon
              className="icon"
              onClick={() => {
                // eslint-disable-next-line no-unused-expressions
                handleChangeRegion({ isLocked: false })
              }}
            />
          )}
        </Tooltip>
      }
      visible={
        <Tooltip title="Show/Hide" placement="right">
          {visible.length ? (
            <VisibleIcon
              className="icon"
              onClick={() => {
                // eslint-disable-next-line no-unused-expressions
                handleChangeRegion({ isHide: true })
              }}
            />
          ) : (
            <VisibleOffIcon
              className="icon"
              onClick={() => {
                // eslint-disable-next-line no-unused-expressions
                handleChangeRegion({ isHide: false })
              }}
            />
          )}
        </Tooltip>
      }
    />
  )
}

const MemoRowHeader = RowHeader

const Row = ({
  region: r,
  highlighted,
  color,
  index,
  delete_annotation,
  setdelete_annotation,
  brushLines = [],
  setLines = () => {},
}) => {
  const handleLockUnLock = (locked, lock_id) => {
    let linessave = [...brushLines]
    let filter_arr = linessave.filter((filters) => {
      if (filters.id === lock_id) {
        filters.isLocked = locked
      }
      return filters
    })
    setLines(filter_arr)
  }
  const handleShowHide = (show, show_id) => {
    let linessave = [...brushLines]
    let filter_arr = linessave.filter((filters) => {
      if (filters.id === show_id) {
        filters.isHide = show
      }
      return filters
    })
    setLines(filter_arr)
  }
  return (
    <RowLayout
      header={false}
      highlighted={highlighted}
      order={`#${index + 1}`}
      classification={
        <Chip
          text={r?.popUp?.classification?.label || ""}
          color={color || "#ddd"}
          index={index}
        />
      }
      area=""
      trash={
        <Tooltip title="Delete" placement="right">
          <TrashIcon
            onClick={() => {
              let linessave = [...brushLines]
              let filter_arr = linessave.filter(
                (filters) => filters.id !== r.id && filters
              )
              setLines(filter_arr)
              let arr = [...delete_annotation]
              let delete_arr = linessave.filter(
                (filter) => filter.id === r.id && filter
              )
              arr.push(delete_arr)
              window.undoArray = [...window.undoArray, "brush"]
              let obj = { brush: delete_arr }
              let newArr = [...window.annotation_redo]
              if (obj.brush !== undefined) {
                newArr.push(obj)
                window.annotation_redo = newArr
              }
              setdelete_annotation(arr.flat())
              window.brushHighlighted(-1)
            }}
            className="icon2"
          />
        </Tooltip>
      }
      lock={
        r.isLocked ? (
          <Tooltip title="UnLock" placement="right">
            <LockIcon
              onClick={() => {
                handleLockUnLock(false, r.id)
                // window.brushHighlighted(index)
              }}
              className="icon2"
            />
          </Tooltip>
        ) : (
          <Tooltip title="Lock" placement="right">
            <UnlockIcon
              onClick={() => {
                handleLockUnLock(true, r.id)
                window.brushHighlighted(-1)
              }}
              className="icon2"
            />
          </Tooltip>
        )
      }
      visible={
        r.isHide || r.isHide === undefined ? (
          <Tooltip title="Show" placement="right">
            <VisibleOffIcon
              onClick={() => {
                handleShowHide(false, r.id)
                window.brushHighlighted(index)
              }}
              className="icon2"
            />
          </Tooltip>
        ) : (
          <Tooltip title="Hide" placement="right">
            <VisibleIcon
              onClick={() => {
                handleShowHide(true, r.id)
                window.brushHighlighted(-1)
              }}
              className="icon2"
            />
          </Tooltip>
        )
      }
    />
  )
}
const MemoRow = memo(Row)

const BrushRegion = ({
  setdelete_annotation,
  onChangeVideoTime,
  setLines,
  brushLines,
  delete_annotation,
  brushHighlight,
}) => {
  const classes = useStyles()
  let brush_region_arr = brushLines?.filter(
    (arr, index, self) =>
      index === self.findIndex((t) => t.id === arr.id && arr.id)
  )
  return (
    <ThemeProvider theme={theme}>
      <SidebarBoxContainer
        title="Brush Region"
        subTitle=""
        icon={<Brush style={{ color: grey[700] }} />}
        expandedByDefault
      >
        <div className={classes.container}>
          <MemoRowHeader
            regions={brush_region_arr}
            // onChangeRegion={onChangeRegion}
            // onDeleteRegion={onDeleteRegion}
            delete_annotation={delete_annotation}
            setdelete_annotation={setdelete_annotation}
            brushLines={brushLines}
            setLines={setLines}
          />
          <HeaderSep />
          {brush_region_arr.map((r, i) => {
            return (
              <MemoRow
                key={r.id}
                {...r}
                region={r}
                index={i}
                delete_annotation={delete_annotation}
                setdelete_annotation={setdelete_annotation}
                setLines={setLines}
                brushLines={brushLines}
              />
            )
          })}
        </div>
      </SidebarBoxContainer>
    </ThemeProvider>
  )
}

export default BrushRegion

