import { useEffect, useState } from "react"

export const useKeyPressed = (targetKey) => {
  const [keyPressed, setKeyPressed] = useState(false)

  const downHandler = ({ key, ctrlKey, altKey }) => {
    if (ctrlKey && altKey && key === targetKey) setKeyPressed(true)
  }

  const upHandler = ({ key, ctrlKey, altKey }) => {
    if (ctrlKey && altKey && key === targetKey) setKeyPressed(false)
  }

  useEffect(() => {
    window.addEventListener("keydown", downHandler)
    window.addEventListener("keyup", upHandler)

    return () => {
      window.removeEventListener("keydown", downHandler)
      window.removeEventListener("keyup", upHandler)
    }
  }, [])

  return keyPressed
}

